@@ -20,7 +20,8 @@ class JsRoutes
     camel_case: false,
     default_url_options: {},
     compact: false,
-    serializer: nil
+    serializer: nil,
+    special_options_key: "_options",
   }
 
   NODE_TYPES = {
@@ -35,7 +36,8 @@ class JsRoutes
   }
 
   LAST_OPTIONS_KEY = "options".freeze
-  FILTERED_DEFAULT_PARTS = [:controller, :action]
+  FILTERED_DEFAULT_PARTS = [:controller, :action, :subdomain]
+  URL_OPTIONS = [:protocol, :domain, :host, :port, :subdomain]
 
   class Options < Struct.new(*DEFAULTS.keys)
     def to_hash
@@ -104,10 +106,11 @@ class JsRoutes
       "APP_CLASS"           => Rails.application.class.to_s,
       "NAMESPACE"           => @options[:namespace],
       "DEFAULT_URL_OPTIONS" => json(@options[:default_url_options].merge(deprecate_url_options)),
-      "PREFIX"              => @options[:prefix] || "",
+      "PREFIX"              => @options[:prefix] || Rails.application.config.relative_url_root || "",
       "NODE_TYPES"          => json(NODE_TYPES),
-      "SERIALIZER"          => @options[:serializer] || "null",
+      "SERIALIZER"          => @options[:serializer] || json(nil),
       "ROUTES"              => js_routes,
+      "SPECIAL_OPTIONS_KEY" => @options[:special_options_key].to_s
     }.inject(File.read(File.dirname(__FILE__) + "/routes.js")) do |js, (key, value)|
       js.gsub!(key, value)
     end
@@ -150,20 +153,23 @@ class JsRoutes
   protected
 
   def js_routes
-    js_routes = Rails.application.routes.named_routes.to_a.sort_by(&:to_s).flat_map do |_, route|
-      rails_engine_app = get_app_from_route(route)
-      if rails_engine_app.respond_to?(:superclass) && rails_engine_app.superclass == Rails::Engine && !route.path.anchored
-        rails_engine_app.routes.named_routes.map do |_, engine_route|
-          build_route_if_match(engine_route, route)
-        end
-      else
-        build_route_if_match(route)
-      end
+    js_routes = Rails.application.routes.named_routes.to_a.sort_by(&:first).flat_map do |_, route|
+      [build_route_if_match(route)] + mounted_app_routes(route)
     end.compact
-
     "{\n" + js_routes.join(",\n") + "}\n"
   end
 
+  def mounted_app_routes(route)
+    rails_engine_app = get_app_from_route(route)
+    if rails_engine_app.respond_to?(:superclass) && rails_engine_app.superclass == Rails::Engine && !route.path.anchored
+      rails_engine_app.routes.named_routes.map do |_, engine_route|
+        build_route_if_match(engine_route, route)
+      end
+    else
+      []
+    end
+  end
+
   def get_app_from_route(route)
     # rails engine in Rails 4.2 use additional ActionDispatch::Routing::Mapper::Constraints, which contain app
     if route.app.respond_to?(:app) && route.app.respond_to?(:constraints)
@@ -204,8 +210,8 @@ class JsRoutes
   def route_js_arguments(route, parent_spec)
     required_parts = route.required_parts.clone
     optional_parts = route.parts - required_parts
-    default_parts = route.defaults.reject do |part, _|
-      FILTERED_DEFAULT_PARTS.include?(part)
+    default_parts = route.defaults.select do |part, _|
+      FILTERED_DEFAULT_PARTS.exclude?(part) && URL_OPTIONS.include?(part) || required_parts.include?(part)
     end
     [
       required_parts, optional_parts, serialize(route.path.spec, parent_spec), default_parts
