@@ -186,9 +186,18 @@ describe JsRoutes, "options" do
 
   describe "default_url_options" do
     context "with optional route parts" do
-      let(:_options) { {:default_url_options => {:optional_id => "12", :format => "json"}}}
-      it "should use this opions to fill optional parameters" do
-        expect(evaljs("Routes.things_path()")).to eq(routes.things_path(:optional_id => 12, :format => "json"))
+      context "provided" do
+        let(:_options) { { :default_url_options => { :optional_id => "12", :format => "json" } } }
+        it "should use this opions to fill optional parameters" do
+          expect(evaljs("Routes.things_path()")).to eq(routes.things_path(:optional_id => 12, :format => "json"))
+        end
+      end
+
+      context "not provided" do
+        let(:_options) { { :default_url_options => { :format => "json" } } }
+        it "breaks" do
+          expect(evaljs("Routes.foo_all_path()")).to eq(routes.foo_all_path(:format => "json"))
+        end
       end
     end
 
@@ -412,14 +421,7 @@ describe JsRoutes, "options" do
       end
 
       before do
-        jscontext['window'] = {
-          :location => {
-            :protocol => current_protocol,
-            :hostname => current_hostname,
-            :port => current_port,
-            :host => current_host
-          }
-        }
+        jscontext.eval("window = {'location': {'protocol': '#{current_protocol}', 'hostname': '#{current_hostname}', 'port': '#{current_port}', 'host': '#{current_host}'}}")
       end
 
       context "without specifying a default host" do
@@ -474,4 +476,14 @@ describe JsRoutes, "options" do
       end
     end
   end
+
+  describe "special_options_key" do
+    let(:_options) { { special_options_key: :__options__ } }
+    it "can be redefined" do
+      expect {
+        expect(evaljs("Routes.inbox_message_path({inbox_id: 1, id: 2, _options: true})")).to eq("")
+      }.to raise_error(js_error_class)
+      expect(evaljs("Routes.inbox_message_path({inbox_id: 1, id: 2, __options__: true})")).to eq(routes.inbox_message_path(inbox_id: 1, id: 2))
+    end
+  end
 end
